import { useDocuments, useFolders } from '../hooks/swrHooks'
import { IDocument, IDocumentFolder } from '../interfaces/documents'
import { Box, Breadcrumbs, Button, Card, CardActionArea, CircularProgress, Divider, Input, InputLabel, LinearProgress, Link, List, ListItem, ListItemButton } from '@mui/material'
import { Download, Folder, InsertDriveFile, Upload, UploadFile } from '@mui/icons-material'
import { useEffect, useRef, useState } from 'react'
import DocumentService from '../services/DocumentService'
import { mutate } from 'swr'
import FileViewer from './modals/FileViewer'
import { fileTypeFromBlob } from 'file-type/core'

interface FolderProps {
    folder: IDocumentFolder;
    handleFolderClick: (folder: IDocumentFolder) => void;
}

interface DocumentProps {
    doc: IDocument;
    index: number;
    handleDocumentClick: (doc: IDocument, index: number) => void;
}

function ItemFolder({ folder, handleFolderClick, ...props }: FolderProps) {
    return (
        <ListItemButton
            onClick={() => handleFolderClick(folder)}
        >
            <Box
                sx={{
                    cursor: 'pointer',
                    display: 'flex',
                    flexDirection: 'row',
                    gap: '8px',
                    alignItems: 'center',
                    padding: '8px'
                }}
                {...props}
            >
                <Folder />
                {folder.name}
            </Box>
        </ListItemButton>
    )
}

function ItemDocument({ doc, index, handleDocumentClick, ...props }: DocumentProps) {
    return (
        <ListItemButton
            onClick={() => handleDocumentClick(doc, index)}
        >
            <Box
                sx={{
                    cursor: 'pointer',
                    display: 'flex',
                    flexDirection: 'row',
                    gap: '8px',
                    alignItems: 'center',
                    padding: '8px',
                }}
                {...props}
            >
                <InsertDriveFile />
                {doc.name}
            </Box>
        </ListItemButton>
    )
}

export default function FolderViewer() {
    const [currentFolder, setCurrentFolder] = useState<IDocumentFolder | null>(null)

    const [breadcrumbs, setBreadcrumbs] = useState<IDocumentFolder[]>([])

    const { folders, isLoading: foldersLoading } = useFolders()

    const { documents, isLoading: documentsLoading } = useDocuments(currentFolder?.id)

    const [uploadProgress, setUploadProgress] = useState(0)

    const [isUploading, setIsUploading] = useState(false)

    const fileInputRef = useRef<HTMLInputElement>(null)

    const [fileViewerModal, setFileViewerModal] = useState(false)

    const [currentFileNo, setCurrentFileNo] = useState<number>(-1)

    const handleFolderClick = (folder: IDocumentFolder) => {
        setCurrentFolder(folder)
        setBreadcrumbs((prev) => [...prev, folder])
    }

    const handleDocumentClick = async (doc: IDocument, index: number) => {
        setCurrentFileNo(index)
        setFileViewerModal(true)
    }

    const handleBreadcrumbClick = (index: number) => {
        const newBreadcrumbs = breadcrumbs.slice(0, index + 1);
        setBreadcrumbs(newBreadcrumbs)
        setCurrentFolder(newBreadcrumbs[newBreadcrumbs.length - 1])
    }

    const handleFileUpload = async (event: any) => {
        setIsUploading(true)
        const file = event.target.files?.[0]

        if (file && currentFolder && currentFolder.id) {
            const formData = new FormData()
            formData.append('files', file)

            try {
                const response = await DocumentService.uploadFiles(currentFolder.id, formData, setUploadProgress);
                setIsUploading(false);
                mutate(`/info/documents/${currentFolder.id}`);
            } catch (error) {
                console.error(error);
                setIsUploading(false);
            }
        }
    }

    const handleUploadClick = () => {
        if (fileInputRef.current) {
            fileInputRef.current.click();
        }
    }

    if (foldersLoading || documentsLoading) {
        return (
            <CircularProgress />
        )
    }

    return (
        <Box sx={{
            display: 'flex',
            flexDirection: 'column',
            gap: '24px'
        }}>
            <FileViewer
                open={fileViewerModal}
                setOpen={setFileViewerModal}
                currentFileNo={currentFileNo}
                setCurrentFileNo={setCurrentFileNo}
                docs={documents}
            />

            <Breadcrumbs>
                <Link
                    underline='hover'
                    color='inherit'
                    onClick={() => {
                        setCurrentFolder(null)
                        setBreadcrumbs([])
                    }}
                    sx={{ cursor: 'pointer' }}
                >
                    Главная
                </Link>

                {breadcrumbs.map((breadcrumb, index) => (
                    <Link
                        key={breadcrumb.id}
                        underline="hover"
                        color="inherit"
                        onClick={() => handleBreadcrumbClick(index)}
                        sx={{ cursor: 'pointer' }}
                    >
                        {breadcrumb.name}
                    </Link>
                ))}
            </Breadcrumbs>

            <Box>
                {currentFolder &&
                    <Button
                        LinkComponent="label"
                        role={undefined}
                        variant="outlined"
                        tabIndex={-1}
                        startIcon={
                            isUploading ? <CircularProgress sx={{ maxHeight: "20px", maxWidth: "20px" }} variant="determinate" value={uploadProgress} /> : <UploadFile />
                        }
                        onClick={handleUploadClick}
                    >
                        <input
                            type='file'
                            ref={fileInputRef}
                            style={{ display: 'none' }}
                            onChange={handleFileUpload}
                        />
                        Загрузить
                    </Button>
                }
            </Box>

            <List
                dense
            >
                {currentFolder ? (
                    documents?.map((doc: IDocument, index: number) => (
                        <div key={`${doc.id}-${doc.name}`}>
                            <ItemDocument
                                doc={doc}
                                index={index}
                                handleDocumentClick={handleDocumentClick}
                            />
                            {index < documents.length - 1 && <Divider />}
                        </div>

                    ))
                ) : (
                    folders?.map((folder: IDocumentFolder, index: number) => (
                        <div key={`${folder.id}-${folder.name}`}>
                            <ItemFolder
                                folder={folder}
                                handleFolderClick={handleFolderClick}
                            />
                            {index < folders.length - 1 && <Divider />}
                        </div>
                    ))
                )}
            </List>
        </Box>
    )
}